"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const antlr4ts_1 = require("antlr4ts");
const ParseTreeWalker_1 = require("antlr4ts/tree/ParseTreeWalker");
const RPScriptParser_1 = require("../antlr/grammar/RPScriptParser");
const RPScriptLexer_1 = require("../antlr/grammar/RPScriptLexer");
const RpsListener_1 = require("../antlr/RpsListener");
const RpsErrorHandling_1 = require("../antlr/RpsErrorHandling");
const ts_deferred_1 = require("ts-deferred");
const events_1 = require("events");
var _eval = require('../../lib/eval');
const rpscript_interface_1 = require("rpscript-interface");
const modulemgr_1 = require("./modulemgr");
const shelljs_1 = __importDefault(require("shelljs"));
const js_yaml_min_1 = __importDefault(require("../../lib/js-yaml.min"));
const __1 = require("..");
class Runner extends events_1.EventEmitter {
    constructor(config) {
        super();
        this.config = config;
        this.wordMgr = new __1.KeywordsMgr;
    }
    execute(filepath, args, content) {
        return __awaiter(this, void 0, void 0, function* () {
            this.emit(Runner.COMPILE_START_EVT, filepath);
            let rpsContent = filepath ? fs_1.default.readFileSync(filepath, 'utf8') : content;
            let transpileContent;
            try {
                transpileContent = yield this.transpile(filepath, rpsContent);
            }
            catch (err) {
                this.emit(Runner.TRANSPILE_ERR_EVT, err);
                return {};
            }
            let tsContent = transpileContent.fullContent;
            let verbs = transpileContent.verbs;
            this.emit(Runner.COMPILED_EVT, { transpile: tsContent });
            if (!this.config.skipRun)
                this.run(tsContent, verbs, args);
            return { transpile: transpileContent };
        });
    }
    //this is for debug purpose only
    exeFromSource(fileName) {
        return __awaiter(this, void 0, void 0, function* () {
            let output = fs_1.default.readFileSync(fileName, 'utf8');
            let modMgr = new modulemgr_1.ModuleMgr;
            let rpsContext = this.initRpsContext(this.config.configFilesLocation, []);
            let modulesToBeLoaded = modMgr.listModuleNames();
            let context = yield modMgr.loadModuleObjs(rpsContext, modulesToBeLoaded);
            context['EventEmitter'] = events_1.EventEmitter;
            context['$CONTEXT'] = rpsContext;
            this.setupModulesContext(context['$CONTEXT']); //calling modules' setup lifecycle
            yield _eval(output, context, true);
        });
    }
    transpile(filepath, filecontent) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = yield this.convertToTS(filepath, filecontent);
            this.emit(Runner.TRANSPILE_EVT, result);
            return result;
        });
    }
    run(tsContent, verbs, args) {
        return __awaiter(this, void 0, void 0, function* () {
            let context = yield this.initializeContext(verbs, args); //loading modules
            this.setupModulesContext(context['$CONTEXT']); //calling modules' setup lifecycle
            this.runnerListener = yield _eval(tsContent, context, true);
            this.runnerListener.on(Runner.START_EVT, (...params) => this.emit(Runner.START_EVT, params));
            this.runnerListener.on(Runner.END_EVT, (...params) => this.emit(Runner.END_EVT, params));
            this.runnerListener.on(Runner.ACTION_EVT, (...params) => this.emit(Runner.ACTION_EVT, params));
            return this.runnerListener;
        });
    }
    setupModulesContext(ctx) {
        let allCtx = ctx.getAllContexts();
        allCtx.forEach(ctx => {
            if (ctx['setup'])
                ctx['setup'](ctx);
        });
    }
    initializeContext(verbs, args = []) {
        return __awaiter(this, void 0, void 0, function* () {
            let modMgr = new modulemgr_1.ModuleMgr;
            modMgr.event.on(Runner.MOD_LOADED_EVT, (...params) => this.emit(Runner.MOD_LOADED_EVT, params));
            modMgr.event.on(Runner.MOD_DISABLED_EVT, (...params) => this.emit(Runner.MOD_DISABLED_EVT, params));
            let rpsContext = this.initRpsContext(this.config.configFilesLocation, args);
            let modulesToBeLoaded = this.determineModulesToLoad(verbs);
            let context = yield modMgr.loadModuleObjs(rpsContext, modulesToBeLoaded);
            context['EventEmitter'] = events_1.EventEmitter;
            context['$CONTEXT'] = rpsContext;
            rpsContext.event.on(Runner.CTX_PRIOR_SET_EVT, (...params) => this.emit(Runner.CTX_PRIOR_SET_EVT, params));
            return context;
        });
    }
    determineModulesToLoad(verbs) {
        let modules = this.config.modules;
        if (!modules) {
            modules = this.wordMgr.getKeywordsModules(verbs);
        }
        return modules;
    }
    //config convention: rps-<module-id>.yaml
    initRpsContext(configLoc, args) {
        let ctx = new rpscript_interface_1.RpsContext();
        ctx['env'] = args; // ctx.setEnvArgs(args);
        let configPath = configLoc ? configLoc : process.cwd();
        let pathfiles = shelljs_1.default.ls(configPath);
        pathfiles.forEach(file => {
            if (file.trim().startsWith('rps-')) {
                let moduleCtx = js_yaml_min_1.default.safeLoad(fs_1.default.readFileSync(configPath + '/' + file, 'utf8'));
                let moduleId = file.substring(file.lastIndexOf("rps-") + 4, file.lastIndexOf("."));
                ctx.addModuleContext(moduleId, moduleCtx);
            }
        });
        return ctx;
    }
    convertToTS(filepath, content) {
        let d = new ts_deferred_1.Deferred();
        let lexer = this.setupLexer(content);
        let parser = this.setupParser(lexer);
        let intentListener = new RpsListener_1.RpsTranspileListener(d, filepath, parser, this.config.skipKeywordCheck);
        let context = parser.program();
        ParseTreeWalker_1.ParseTreeWalker.DEFAULT.walk(intentListener, context);
        return d.promise;
    }
    setupLexer(content) {
        let inputStream = new antlr4ts_1.ANTLRInputStream(content);
        // let lexer = new RpsTranspileLexer(inputStream);
        let lexer = new RPScriptLexer_1.RPScriptLexer(inputStream);
        lexer.removeErrorListeners();
        return lexer;
    }
    setupParser(lexer) {
        let tokenStream = new antlr4ts_1.CommonTokenStream(lexer);
        let parser = new RPScriptParser_1.RPScriptParser(tokenStream);
        parser.removeErrorListeners();
        parser.addErrorListener(new RpsErrorHandling_1.ErrorCollectorListener);
        return parser;
    }
}
Runner.COMPILE_START_EVT = "runner.compile.start";
Runner.TRANSPILE_EVT = "runner.transpile";
Runner.TRANSPILE_ERR_EVT = "runner.transpile.err";
Runner.LINT_EVT = "runner.linted";
Runner.COMPILED_EVT = "runner.compiled";
Runner.MOD_LOADED_EVT = "runner.module.loaded";
Runner.MOD_DISABLED_EVT = "runner.module.disabled";
Runner.START_EVT = "runner.start";
Runner.END_EVT = "runner.end";
Runner.ACTION_EVT = "action";
Runner.CTX_PRIOR_SET_EVT = "context.priority.set";
exports.Runner = Runner;
